--- old/usr/src/cmd/beadm/beadm.c	Sat Nov 23 11:10:38 2013
+++ new/usr/src/cmd/beadm/beadm.c	Sat Nov 23 11:10:38 2013
@@ -1269,6 +1269,10 @@
 		(void) fprintf(stderr, _("You have insufficient privileges to "
 		    "execute this command.\n"));
 		break;
+	case BE_ERR_NO_MOUNTED_ZONE:
+		(void) fprintf(stderr, _("Mounted on '%s'.\nUnable to mount "
+		    "one of %s's zone BE's.\n"), mountpoint, obe_name);
+		break;
 	default:
 		(void) fprintf(stderr, _("Unable to mount %s.\n"), obe_name);
 		(void) fprintf(stderr, "%s\n", be_err_to_str(err));
--- old/usr/src/lib/libbe/common/be_mount.c	Sat Nov 23 11:10:39 2013
+++ new/usr/src/lib/libbe/common/be_mount.c	Sat Nov 23 11:10:39 2013
@@ -382,12 +382,8 @@
 	if (getzoneid() == GLOBAL_ZONEID &&
 	    !(flags & BE_MOUNT_FLAG_NO_ZONES) &&
 	    be_get_uuid(bt.obe_root_ds, &uu) == BE_SUCCESS) {
-		if ((ret = be_mount_zones(zhp, &md)) != BE_SUCCESS) {
-			(void) _be_unmount(bt.obe_name, 0);
-			if (gen_tmp_altroot)
-				free(tmp_altroot);
-			ZFS_CLOSE(zhp);
-			return (ret);
+		if (be_mount_zones(zhp, &md) != BE_SUCCESS) {
+			ret = BE_ERR_NO_MOUNTED_ZONE;
 		}
 	}
 
@@ -397,10 +393,14 @@
 	 * If a NULL altroot was passed in, pass the generated altroot
 	 * back to the caller in altroot.
 	 */
-	if (gen_tmp_altroot)
-		*altroot = tmp_altroot;
+	if (gen_tmp_altroot) {
+		if (ret == BE_SUCCESS || ret == BE_ERR_NO_MOUNTED_ZONE)
+			*altroot = tmp_altroot;
+		else
+			free(tmp_altroot);
+	}
 
-	return (BE_SUCCESS);
+	return (ret);
 }
 
 /*
@@ -1712,10 +1712,7 @@
 /*
  * Function:	loopback_mount_zonepath
  * Description:	This function loopback mounts a zonepath into the altroot
- *		area of the BE being mounted.  Since these are shared file
- *		systems, they are expected to be already mounted for the
- *		current BE, and this function just loopback mounts them into
- *		the BE mountpoint.
+ *		area of the BE being mounted.
  * Parameters:
  *		zonepath - pointer to zone path in the current BE
  *		md - be_mount_data_t pointer
--- old/usr/src/man/man1m/beadm.1m	Sat Nov 23 11:10:39 2013
+++ new/usr/src/man/man1m/beadm.1m	Sat Nov 23 11:10:39 2013
@@ -1,6 +1,6 @@
 '\" te
 .\" Copyright 2013 Nexenta Systems, Inc. All rights reserved.
-.TH BEADM 1M "Jul 25, 2013"
+.TH BEADM 1M "Nov 11, 2013"
 .SH NAME
 beadm \- utility for managing zfs boot environments
 .SH SYNOPSIS
@@ -29,7 +29,7 @@
 
 .LP
 .nf
-\fBbeadm\fR \fBmount\fR [\fB-v\fR] \fIbeName\fR \fImountpoint\fR
+\fBbeadm\fR \fBmount\fR [\fB-s\fR \fBro\fR|\fBrw\fR] [\fB-v\fR] \fIbeName\fR \fImountpoint\fR
 .fi
 
 .LP
@@ -407,7 +407,7 @@
 .sp
 .ne 2
 .na
-\fBbeadm\fR \fBmount\fR [\fB-v\fR] \fIbeName\fR \fImountpoint\fR
+\fBbeadm\fR \fBmount\fR [\fB-s\fR \fBro\fR|\fBrw\fR] [\fB-v\fR] \fIbeName\fR \fImountpoint\fR
 .ad
 .sp .6
 .RS 4n
@@ -415,6 +415,15 @@
 already existing empty directory.
 .sp
 .ne 2
+.na
+\fB-s\fR \fBro\fR|\fBrw\fR
+.ad
+.sp .6
+.RS 4n
+Mount the shared filesystems of the BE in read-only or read-write mode.
+.RE
+.sp
+.ne 2
 .na
 \fB-v\fR
 .ad
